/* n-event.js by nanto_vi
 *
 * イベントリスナの操作を補助する。
 *
 * 使い方:
 *
 *   var listener = new NEvent(element, "click", func);
 *     elementのclickイベントにfunc関数をリスナとして登録する。
 *
 *   var listener = new NEvent(element, "click", obj.method, obj);
 *     elementのclickイベントにobjオブジェクトの
 *     methodメソッドをリスナとして登録する。
 *
 *   var listener = new NEvent(element, "click", "method", obj);
 *     elementのclickイベントにobjオブジェクトの
 *     methodメソッドをリスナとして登録する。
 *
 *   listener.unlisten();
 *     リスナの登録を解除する。
 *
 *   listener.listen();
 *     リスナを再登録する。
 *
 * イベント名には以下の独自イベントを指定可能。
 *
 *   ne_focusin
 *       focusイベントと同じだが、子孫要素のfocusイベントにも反応する。
 *   ne_focusout
 *       blurイベントと同じだが、子孫要素のblurイベントにも反応する。
 *   ne_keydown
 *       keydownイベントと同じだが、Operaではkeypressイベントに反応する。
 */

function NEvent(target, type, method, object) {
  var handler = NEvent.getHandler(object, method, target);
  var useCapture = false;
  if (type.indexOf("ne_") == 0) {
    useCapture = type in NEvent.useCaptureMap;
    type = NEvent.typeMap[type] || type.substring(3);
  }
  this.target = target;
  this.type = type;
  this.handler = handler;
  this.useCapture = useCapture;
  this.listen();
};

NEvent.prototype.listen = function () {
  this.target.addEventListener(this.type, this.handler, this.useCapture);
};

NEvent.prototype.unlisten = function () {
  this.target.removeEventListener(this.type, this.handler, this.useCapture);
};

NEvent.getHandler = function (object, method) {
  if (!object) return method;
  if (typeof method == "string")
    method = object[method];
  return function (event) {
    method.call(object, event);
  };
};

NEvent.typeMap = {
  ne_focusin: "focus",
  ne_focusout: "blur"
};

NEvent.useCaptureMap = {
  ne_focusin: true,
  ne_focusout: true
};

if (navigator.userAgent.indexOf("Opera") != -1)
  NEvent.typeMap.ne_keydown = "keypress";

if (!document.addEventListener && document.attachEvent) {
  NEvent.prototype.listen = function () {
    this.target.attachEvent("on" + this.type, this.handler);
  };

  NEvent.prototype.unlisten = function () {
    this.target.detachEvent("on" + this.type, this.handler);
  };

  NEvent.getHandler = function (object, method, currentTarget) {
    if (!object) {
      if (typeof method.handleEvent == "function") {
        object = method;
        method = object.handleEvent;
      } else {
        object = currentTarget;
      }
    } else if (typeof method == "string") {
      method = object[method];
    }
    return function (event) {
      event.target = event.srcElement;
      event.currentTarget = currentTarget;
      event.stopPropagetion = NEvent.stopPropagetion;
      event.preventDefault = NEvent.preventDefault;
      method.call(object, event);
    };
  };

  NEvent.stopPropagetion = function () {
    this.cancelBubble = true;
  };

  NEvent.preventDefault = function () {
    this.returnValue = false;
  };

  NEvent.typeMap = {};
}
